@@ -16,11 +16,14 @@ pub mod receive_result;
 pub mod pool;
 pub mod unreliable_sender;
 
+mod connection_impl;
+
 // additional stress/scale testing
 #[cfg(test)]
 pub mod tachyon_test;
 
-use std::collections::VecDeque;
+use std::time::Duration;
+use std::time::Instant;
 
 use rustc_hash::FxHashMap;
 
@@ -45,9 +48,12 @@ pub const SEND_ERROR_SOCKET: u32 = 1;
 pub const SEND_ERROR_FRAGMENT: u32 = 3;
 pub const SEND_ERROR_UNKNOWN: u32 = 4;
 pub const SEND_ERROR_LENGTH: u32 = 5;
+pub const SEND_ERROR_IDENTITY: u32 = 6;
 
 pub static mut NONE_SEND_DATA: &'static mut [u8] = &mut [0; TACHYON_HEADER_SIZE];
 
+pub type OnConnectedCallback = unsafe extern "C" fn();
+
 #[derive(Clone, Copy)]
 #[repr(C)]
 #[derive(Default)]
@@ -56,28 +62,29 @@ pub struct TachyonStats {
     pub channel_stats: ChannelStats,
     pub packets_dropped: u64,
     pub unreliable_sent: u64,
-    pub unreliable_received: u64,
-    pub code_time: u64
+    pub unreliable_received: u64
 }
 
 impl std::fmt::Display for TachyonStats {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        write!(f,"channel_stats:{0} packets_dropped:{1} unreliable_sent:{2} unreliable_received:{3} code_time:{4}\n",
-         self.channel_stats, self.packets_dropped, self.unreliable_sent, self.unreliable_received, self.code_time)
+        write!(f,"channel_stats:{0} packets_dropped:{1} unreliable_sent:{2} unreliable_received:{3}\n",
+         self.channel_stats, self.packets_dropped, self.unreliable_sent, self.unreliable_received)
     }
 }
 
 #[derive(Clone, Copy)]
 #[repr(C)]
 pub struct TachyonConfig {
+    pub use_identity: u32,
     pub drop_packet_chance: u64,
-    pub drop_reliable_only: u8,
+    pub drop_reliable_only: u32,
     pub receive_window_size: u16
 }
 
 impl  TachyonConfig {
     pub fn default() -> Self {
         let default = TachyonConfig {
+            use_identity: 0,
             drop_packet_chance: 0,
             drop_reliable_only: 0,
             receive_window_size: RECEIVE_WINDOW_SIZE_DEFAULT
@@ -107,13 +114,18 @@ pub struct Tachyon {
     pub id: u16,
     pub socket: TachyonSocket,
     pub unreliable_sender: Option<UnreliableSender>,
+    pub identities: FxHashMap<u32, u32>,
     pub connections: FxHashMap<NetworkAddress, Connection>,
     pub channels: FxHashMap<(NetworkAddress,u8),Channel>,
     pub channel_config: FxHashMap<u8, bool>,
     pub config: TachyonConfig,
     pub nack_send_data: Vec<u8>,
     pub resend_sequences: Vec<u16>,
-    pub stats: TachyonStats
+    pub stats: TachyonStats,
+    pub start_time: Instant,
+    pub last_identity_link_request: Instant,
+    pub identity: Identity,
+    pub on_connected_callback: Option<OnConnectedCallback>
 }
 
 impl Tachyon {
@@ -127,6 +139,7 @@ impl Tachyon {
       
         let mut tachyon = Tachyon {
             id,
+            identities: FxHashMap::default(),
             connections: FxHashMap::default(),
             channels: FxHashMap::default(),
             channel_config: FxHashMap::default(),
@@ -135,7 +148,11 @@ impl Tachyon {
             config,
             nack_send_data: vec![0;4096],
             resend_sequences: Vec::new(),
-            stats: TachyonStats::default()
+            stats: TachyonStats::default(),
+            start_time: Instant::now(),
+            last_identity_link_request: Instant::now() - Duration::new(100, 0),
+            identity: Identity::default(),
+            on_connected_callback: None
         };
         tachyon.channel_config.insert(1, true);
         tachyon.channel_config.insert(2, false);
@@ -151,6 +168,10 @@ impl Tachyon {
         header.write(unsafe {&mut NONE_SEND_DATA});
     }
 
+    pub fn time_since_start(&self) -> u64 {
+        return Instant::now().duration_since(self.start_time).as_millis() as u64;
+    }
+
     pub fn bind(&mut self, address: NetworkAddress) -> bool {
         match self.socket.bind_socket(address) {
             CreateConnectResult::Success => {
@@ -189,15 +210,6 @@ impl Tachyon {
         return Some(sender);
     }
 
-    fn try_create_connection(&mut self, address: NetworkAddress) -> bool {
-        if !self.connections.contains_key(&address) {
-            let conn = Connection::create(address, self.id);
-            self.connections.insert(address, conn);
-            return true;
-        } else {
-            return false;
-        }
-    }
 
     pub fn get_channel(&mut self, address: NetworkAddress, channel_id: u8) -> Option<&mut Channel> {
         match self.channels.get_mut(&(address, channel_id)) {
@@ -208,7 +220,7 @@ impl Tachyon {
         }
     }
 
-    pub fn create_configured_channels(&mut self, address: NetworkAddress) {
+    fn create_configured_channels(&mut self, address: NetworkAddress) {
         for config in &self.channel_config {
             let channel_id = *config.0;
             let ordered = *config.1;
@@ -223,6 +235,24 @@ impl Tachyon {
         }
     }
 
+    pub fn get_channel_count(&mut self, address: NetworkAddress) -> u32 {
+        let mut count = 0;
+        for config in &self.channel_config {
+            let channel_id = *config.0;
+            if self.channels.contains_key(&(address, channel_id)) {
+                count += 1;
+            }
+        }
+        return count;
+    }
+
+    fn remove_configured_channels(&mut self, address: NetworkAddress) {
+        for config in &self.channel_config {
+            let channel_id = *config.0;
+            self.channels.remove(&(address, channel_id));
+        }
+    }
+
     pub fn configure_channel(&mut self, channel_id: u8, ordered: bool) -> bool {
         if channel_id < 3 {
             return false;
@@ -231,9 +261,6 @@ impl Tachyon {
         return true;
     }
 
-    
-    
-
     pub fn get_combined_stats(&mut self) -> TachyonStats {
         let mut channel_stats = ChannelStats::default();
         for channel in self.channels.values_mut() {
@@ -245,23 +272,10 @@ impl Tachyon {
         return stats;
     }
 
-    pub fn get_connections(&self, max: u16) -> Vec<Connection> {
-        let mut list: Vec<Connection> = Vec::new();
-        let result_count = std::cmp::min(self.connections.len(), max as usize);
-
-        let mut count = 0;
     
-        for (address, conn) in &self.connections {
-            list.push(*conn);
-            count += 1;
-            if count >= result_count {
-                break;
-            }
-        }
-        return list;
-    }
-
     pub fn update(&mut self) {
+        self.client_identity_update();
+
         for channel in self.channels.values_mut() {
             channel.frag.expire_groups();
             channel.receiver.publish();
@@ -291,14 +305,19 @@ impl Tachyon {
         }
     }
 
-    fn receive_published_all_channels(&mut self, receive_buffer:  &mut[u8]) -> (u32, NetworkAddress) {
+    fn receive_published_all_channels(&mut self, receive_buffer:  &mut[u8]) -> TachyonReceiveResult {
+        let mut result = TachyonReceiveResult::default();
+
         for channel in self.channels.values_mut() {
             let res = channel.receive_published(receive_buffer);
             if res.0 > 0 {
-                return res;
+                result.length = res.0;
+                result.address = res.1;
+                result.channel = channel.id as u16;
+                return result;
             }
         }
-        return (0, NetworkAddress::default());
+        return result;
     }
 
 
@@ -312,6 +331,7 @@ impl Tachyon {
                 ReceiveResult::Reliable {network_address: socket_addr, channel_id } => {
                     let published = self.receive_published_channel_id(receive_buffer, socket_addr, channel_id);
                     if published > 0 {
+                        result.channel = channel_id as u16;
                         result.length = published;
                         result.address = socket_addr;
                         return result;
@@ -329,15 +349,8 @@ impl Tachyon {
             }
         }
         
-        let published = self.receive_published_all_channels(receive_buffer);
-        if published.0 > 0 {
-            result.length = published.0;
-            result.address = published.1;
-            return result;
-        }
+        return self.receive_published_all_channels(receive_buffer);
         
-
-        return result;
     }
 
     
@@ -345,18 +358,55 @@ impl Tachyon {
         
         let address: NetworkAddress;
         let received_len: usize;
-        
+        let header: Header;
+
         let socket_result = self.socket.receive(receive_buffer, self.config.drop_packet_chance, self.config.drop_reliable_only == 1);
         match socket_result {
             SocketReceiveResult::Success { bytes_received, network_address } => {
                 received_len = bytes_received;
                 address = network_address;
 
+                header = Header::read(receive_buffer);
+                
                 if self.socket.is_server {
-                    if self.try_create_connection(address) {
-                        self.create_configured_channels(address);
+                    if self.config.use_identity == 1 {
+                        let connection_header: ConnectionHeader;
+
+                        if header.message_type == MESSAGE_TYPE_LINK_IDENTITY {
+                            connection_header = ConnectionHeader::read(receive_buffer);
+                            self.try_link_identity(address, connection_header.id, connection_header.session_id);
+                            return ReceiveResult::Empty;
+                        } else if header.message_type == MESSAGE_TYPE_UNLINK_IDENTITY {
+                            connection_header = ConnectionHeader::read(receive_buffer);
+                            self.try_unlink_identity(address,connection_header.id, connection_header.session_id);
+                            return ReceiveResult::Empty;
+                        } else {
+                            if !self.validate_and_update_linked_connection(address) {
+                                return ReceiveResult::Empty;
+                            }
+                        }
+                    } else {
+                        self.on_receive_connection_update(address);
+                    }
+                } else {
+                    if self.config.use_identity == 1 {
+                        if header.message_type == MESSAGE_TYPE_IDENTITY_LINKED {
+                            self.identity.set_linked(1);
+                            if let Some(callback) = self.on_connected_callback {
+                                unsafe {callback();}
+                            }
+                            return ReceiveResult::Empty;
+                        } else if header.message_type == MESSAGE_TYPE_IDENTITY_UNLINKED {
+                            self.identity.set_linked(0);
+                            return ReceiveResult::Empty;
+                        }
+
+                        if !self.identity.is_linked() {
+                            return ReceiveResult::Empty;
+                        }
                     }
                 }
+
             },
             SocketReceiveResult::Empty => {
                 return ReceiveResult::Empty;
@@ -370,7 +420,7 @@ impl Tachyon {
             },
         }
         
-        let header = Header::read(receive_buffer);
+        
        
         if header.message_type == MESSAGE_TYPE_UNRELIABLE {
             self.stats.unreliable_received += 1;
@@ -443,6 +493,12 @@ impl Tachyon {
 
     pub fn send_unreliable(&mut self, address: NetworkAddress, data:  &mut [u8], length: usize) -> TachyonSendResult {
 
+        if !self.can_send() {
+            let mut result = TachyonSendResult::default();
+            result.error = SEND_ERROR_IDENTITY;
+            return result;
+        }
+
         match &self.unreliable_sender {
             Some(sender) => {
                 let result = sender.send_unreliable(address, data, length);
@@ -458,7 +514,6 @@ impl Tachyon {
             },
         }
     }
-
     
     
     pub fn send_reliable(&mut self, channel_id: u8, address: NetworkAddress, data:  &mut [u8], length: usize) -> TachyonSendResult {
@@ -466,6 +521,11 @@ impl Tachyon {
         let message_type = MESSAGE_TYPE_RELIABLE;
         let mut result = TachyonSendResult::default();
 
+        if !self.can_send() {
+            result.error = SEND_ERROR_IDENTITY;
+            return result;
+        }
+
         if length == 0 {
             result.error = SEND_ERROR_LENGTH;
             return result;
@@ -647,14 +707,18 @@ mod tests {
 
         test.receive_buffer[0] = 1;
         test.send_buffer[1] = 4;
-        let sent = test.client_send_unreliable(2);
+        test.send_buffer[2] = 5;
+        test.send_buffer[3] = 6;
+        let sent = test.client_send_unreliable(4);
         assert_eq!(0, sent.error);
-        assert_eq!(2, sent.sent_len as usize);
+        assert_eq!(4, sent.sent_len as usize);
 
         let res = test.server_receive();
-        assert_eq!(2, res.length);
+        assert_eq!(4, res.length);
         assert_eq!(0, test.receive_buffer[0]);
         assert_eq!(4, test.receive_buffer[1]);
+        assert_eq!(5, test.receive_buffer[2]);
+        assert_eq!(6, test.receive_buffer[3]);
     }
 
    
